package com.ruoyi.mybasic.service.Impl;

import com.alibaba.druid.util.StringUtils;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.mybasic.common.domain.Card;
import com.ruoyi.mybasic.common.domain.response.ResponseCard;
import com.ruoyi.mybasic.mapper.CardMapper;
import com.ruoyi.mybasic.service.CardService;
import com.ruoyi.mybasic.util.SdkCardUtils;
import com.ruoyi.mybasic.util.UploadUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.Date;

/**
 * 上传身份证实现层
 * @author Lou_Zs
 */
@Service
public class CardServiceImpl implements CardService {



    @Autowired
    private CardMapper cardMapper;
    @Autowired
    private UploadUtil uploadUtil;
    @Autowired
    private SdkCardUtils sdkCardUtils;

    /**
     * 身份证上传
     * @param fileFace
     * @param fileBack
     */
    @Override
    public void uploadCardFace(MultipartFile fileFace, MultipartFile fileBack) {
        //获取登入id
        Long userId = SecurityUtils.getUserId();
        Card card = cardMapper.findUserCard(userId);
        if (null != card){
            throw new RuntimeException("身份证信息已存在!");
        }
        //正面上传
        String uploadFace = uploadUtil.upload(fileFace);
        if (StringUtils.isEmpty(uploadFace)){
            throw new RuntimeException("正面身份证信息上传失败");
        }
        //将图片文件读取并base64编码
        String img_base64_face = sdkCardUtils.img_base64(uploadFace);
        //反面上传
        //将图片文件读取并base64编码
        String uploadBack= uploadUtil.upload(fileBack);
        if (StringUtils.isEmpty(uploadBack)){
            throw new RuntimeException("反面身份证信息上传失败");
        }
        String img_base64_back = sdkCardUtils.img_base64(uploadBack);

        //提取信息
            //正
        ResponseCard cardedFace = sdkCardUtils.cardFace(img_base64_face);
        if (null == cardedFace){
            throw new RuntimeException("正面身份证信息扫描失败");
        }
            //反
        ResponseCard cardBack = sdkCardUtils.cardBack(img_base64_back);
        if (null == cardBack){
            throw new RuntimeException("反面身份证信息扫描失败");
        }
        //添加身份证信息
        //正
        Card addCard = new Card();
        addCard.setUserId(Math.toIntExact(userId));
        addCard.setCardFace(uploadFace);
        addCard.setCardBack(uploadBack);
        addCard.setNum(cardedFace.getNum());
        addCard.setName(cardedFace.getName());
        addCard.setAddress(cardedFace.getAddress());
        addCard.setSex(cardedFace.getSex());
        addCard.setBirth(cardedFace.getBirth());
        addCard.setNationality(cardedFace.getNationality());
        ///反
        addCard.setStartDate(cardBack.getStartDate());
        addCard.setEndDate(cardBack.getEndDate());
        addCard.setIssue(cardBack.getIssue());
        addCard.setCreateTime(new Date());
        //添加身份证信息
        int i = cardMapper.insertUserCard(addCard);
        if (i < 0){
            throw new RuntimeException("身份证信息填充失败!");
        }
    }


//    /**
//     * 上传身份证正面
//     * @param file
//     */
//    @Override
//    public void uploadCardFace(MultipartFile file) {
//        //获取当前登入人
//        Long userId = SecurityUtils.getUserId();
//        //上传文件
//        String upload = uploadUtil.upload(file);
//        if (StringUtils.isEmpty(upload)){
//            throw new RuntimeException("图片上传失败!");
//        }
//        //查询用户是否存在身份证信息
//        Card card  = cardMapper.findUserCard(userId);
//        if (null != card){
//            //存在身份证信息
//            if (card.getCardFace()!=null && !card.getCardFace().equals("")){
//                //存在身份证正面信息
//                throw new RuntimeException("身份证正面信息已存在!");
//            }
//        }
//        if (null == card){
//            //创建身份证信息表
//            Card addCard = new Card();
//            addCard.setUserId(Math.toIntExact(userId));
//            addCard.setCardFace(upload);
//            int i = cardMapper.insertCard(addCard);
//        }
//        //将图片文件读取并base64编码
//        String img_base64 = sdkCardUtils.img_base64(upload);
//        //读取身份证信息
//        ResponseCard responseCard = sdkCardUtils.cardBack(img_base64);
//        //添加身份证信息详情
//
//    }
}
