package com.ruoyi.mybasic.service.Impl;

import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.mybasic.common.domain.request.ConsultationRequest;
import com.ruoyi.mybasic.common.domain.request.DoctorGiftRecordRequest;
import com.ruoyi.mybasic.common.domain.request.InterrogationRequest;
import com.ruoyi.mybasic.common.domain.response.ConsultationResponse;
import com.ruoyi.mybasic.mapper.FavorableRateMapper;
import com.ruoyi.mybasic.service.FavorableRateService;
import io.swagger.v3.oas.annotations.parameters.RequestBody;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @ClassName FavorableRateServiceImpl
 * @Description 描述
 * @Author WenHao.Sao
 */
@Service
public class FavorableRateServiceImpl implements FavorableRateService {
    /** @noinspection SpringJavaInjectionPointsAutowiringInspection*/
    @Autowired
    private FavorableRateMapper favorableRateMapper;

    /**
     * 根据用户信息获取 问诊记录表 根据问诊记录的状态完成数量查询问诊患者的数量
     * @param userId
     * @return
     */
    @Override
    public R<List<ConsultationRequest>> countPatient(String userId) {

        ConsultationRequest consultation = new ConsultationRequest();
        //首先查询状态字段为一的 0 未完成  1 完成
        consultation.setCount(favorableRateMapper.statusCount(consultation));
        List<ConsultationRequest> consultationList = favorableRateMapper.consultationList(userId);
        R<List<ConsultationRequest>> ok = R.ok(consultationList);

        return ok;
    }

    /**
     * 问诊计数以及查询已经结束问诊的数量
     * @param interrogation
     * @return
     */
    @Override
    public List<InterrogationRequest> interrogationStatus(@RequestBody InterrogationRequest interrogation) {
        return favorableRateMapper.interrogationStatus(interrogation);
    }

    /**
     * 获取到已经结束问诊的好评
     * @param status
     * @return
     */
    @Override
    public List<InterrogationRequest> evaluateCount(InterrogationRequest status) {
        return favorableRateMapper.evaluateCount(status);
    }

    /**
     * 获取到当前登录医生收到的礼物列表
     * 进行礼物记录表以及礼物的表联查
     * @param userId
     * @return
     */
    @Override
    public R<List<DoctorGiftRecordRequest>> giftList(String userId) {
        List<DoctorGiftRecordRequest> giftList = favorableRateMapper.giftList(userId);
        R<List<DoctorGiftRecordRequest>> success = R.ok(giftList);

        return success;
    }
}
